package com.boman.web.core.service.save;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.redis.service.RedisService;
import com.boman.domain.constant.FormDataConstant;
import com.boman.domain.constant.TableNameConst;
import com.boman.web.core.domain.RowResult;
import com.boman.web.core.mapper.StandardMapper;
import com.boman.web.core.utils.IdUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

import static com.boman.common.core.utils.obj.ObjectUtils.*;

/**
 * @author shiqian
 * @description
 * @date 2021年03月22日 11:25
 **/
@Component
public class BaseSaveServiceImpl implements IBaseSaveService {

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseSaveServiceImpl.class);

    @Autowired
    private StandardMapper mapper;
    @Resource
    private RedisService redisService;
    /**
     * 功能描述: 保存一行
     *
     * @param tableName  tableName
     * @param commitData commitData
     * @return com.boman.system.common.RowResult
     */
    @Override
    public RowResult insertRow(String tableName, JSONObject commitData) {
        int ret = mapper.insert(tableName, commitData);
        if (ret > 0) {
            LOGGER.info("保存成功，保存的数据为：{}", commitData);
            commitData.put(FormDataConstant.SUCCESS_CNT, ret);
            return RowResult.ok("保存成功", commitData);
        }

        return RowResult.error("失败");
    }

    @Override
    public int insertList(String tableName, List<JSONObject> datList) {
        requireNonNull(tableName, "insertList tableName is empty");
        requireNonNull(datList, "insertList datList is empty");

        String pkName = IdUtils.getPkName(tableName);

        for (JSONObject data : datList) {
            Long maxId = IdUtils.getMaxId(tableName, pkName);
            data.put(TableNameConst.ID, maxId);
        }

        return mapper.insertList(tableName, datList);
    }
}
